/*
 * Software fr ATMega8 und kompatible
 * fr PWM-Board turi
 * Basissoftware ausgelegt fr:
 *    RC5-Fernbedienung (Port D2)
 *    4 PWM-Kanle mit Helligkeitsregelung fr 8-Bit PWM ber FB
 *    5 ADC-Kanle fr Potis RGB/HSB/wei optional ber Pin-Switch JP1
 * 
 * Software-Komponenten verschiedener Foren verwendet und teilweise adaptiert (s. Doku
 * der jeweiligen Files)
 *
 * Fernbedienung vivanco UR12: TV1:102; bei Bedarf fr 2. Controller SAT1:316
 *
 */


// Komponenten fr RC5 und ADC 
#include "RC5_v1.c"
#include "adc.c"
#include "PWM.c"
#include "lcd-routines.c"
#include "lib_dmx_in.c"


#include <avr/pgmspace.h>
#include <avr/eeprom.h> 
#include <util/delay.h>

//EPROM-Speicher
uint16_t ee_pwm1 EEMEM = 255; //Red
uint16_t ee_pwm2 EEMEM = 127; //Green
uint16_t ee_pwm3 EEMEM = 0; //Blue
uint16_t ee_pwm4 EEMEM = 30; //brightness
uint16_t ee_pwm5 EEMEM = 127; //sat
uint16_t ee_pwm6 EEMEM = 10; //hue
uint8_t ee_pwm7 EEMEM = 9;  //Programm
uint16_t ee_pwm8 EEMEM = 10;  //speed
uint16_t ee_pwm9 EEMEM = 50; //White
uint16_t ee_pwm10 EEMEM = 50; //Strobo_pause
uint16_t ee_pwm11 EEMEM = 50; //Strobo_on
uint16_t ee_pwm12 EEMEM = 1; //DMX-Startadresse
 
uint16_t bri_table_8[64]  PROGMEM = {0,0,1,1,1,2,2,2,2,2,2,3,3,3,3,4,4,4,5,5,6,
									6,7,8,8,9,10,11,12,13,14,15,17,18,20,22,24,
									26,28,31,34,37,40,44,48,52,57,62,68,74,81,89,
									97,106,116,126,138,150,164,179,196,214,234,255};




volatile uint8_t iProg=9;				//Programmart
enum {RGB,HSB,STROBO1,STROBO2,FADE1};	//DMX-Modi
volatile uint8_t iTaste=0;				//Umschalten der Taste nur 1x



//Hilfsfunktion
void delay_ms(uint16_t ms)
{
	for(uint16_t i=0;i<ms;i++) _delay_ms(1);
}



int main (void) {            

	// IR-Sensor Eingang an PD2


	//Konfiguration Jumper Input an PC4 (ADC4 damit blockiert) bei Bedarf
	//Poti enable
	//DDRC &= ~( 1 << PC4 );
	//enable internal pullup PC4
	//PORTC |= 0b00010000;
	//#define Jumper1 PC4

	//Konfiguration Nutzung Input an PB6 (vorauss. kein Quarz) Pin9
	//Poti enable
	DDRB &= ~( 1 << DDB6 );
	//enable internal pullup PB6
	PORTB |= ( 1 << PB6 );
	#define PotiEn PB6

	//Konfiguration Nutzung Taster an PB0 zur man. Programmumschaltung Pin14
	DDRB &= ~( 1 << DDB0 );
	//enable internal pullup PB0
	PORTB |= ( 1 << PB0 );
	#define TA1 PB0;
	//Initialisierung Taster-IR PCINT0
	PCICR |= (1<<PCIE0);				//Pin Change IR 0-7 enabled
	PCMSK0 |=(1<<PCINT0);				//Enable PCINT0

	//Konfiguration Nutzung Input an PB7 (vorauss. kein Quarz) Pin10
	//LCD enable
	DDRB &= ~( 1 << DDB7 );
	//enable internal pullup PB7
	PORTB |= ( 1 << PB7 );
	#define LCDEn PB7

	//DMX-Empfang
	init_DMX_RX();
	uint8_t nDMXmode=RGB;

	

   	//Werte des ADC vor-initialisieren
	uint16_t ADCval; 
	uint16_t ADCvalC1=10; 
	uint16_t ADCvalC2=10;
	uint16_t ADCvalC3=30;
	uint16_t ADCvalC4=10;
	uint16_t ADCvalC0=200;		
	int ADCdiff=0;

	//RC5-Code, Adresse
	uint8_t code, addr;		
	uint8_t DeviceType = TV1;	

	//Variablen fr Farben etc.
	int16_t hue, sat, bri, red, blue, green, white;
	int16_t bri_val, bri_val_old, red_val, green_val, blue_val, white_val;

	uint8_t key=9;				//Taste fr Farbkanalwahl (1,2,3,5); Taste 0 zum Einleiten Speichern
	uint16_t speed=10;			//Fading-Delay
	bri_val_old = 0;			//letzte Helligkeit vor Ausschalten
	uint8_t LEDaus=1;			//Status fr ein/aus
	uint16_t Strobo_pause=50; 	//Pause zwischen dem Aufblitzen
	uint16_t Strobo_on=50; 		//Zeit des Aufblitzens
	uint8_t iRun=0;				//Hilfsvariablr Strobo

	//LCD-Anzeige umschalten
	uint8_t bLCD=0;

	//Aktivierung der Einzelfarbnderung
	uint8_t bEFarbe=0;

	//6 - HSB manuell
	//7 - wei mit Helligkeit
	//8 - RGB manuell
	//9 - HSB-Fading-Programm
	//4 - Strobo
	//5 - DMX-Empfang

	initPWM();

	//Fernbedinung aktivieren
	rc5_init (RC5_ALL);


	/* Interrupts zulassen */
	sei();

	if (! (PINB & (1 << LCDEn)) )
	{
		//LCD initialisieren
		lcd_init();
    	lcd_string("RGBW-Controller");
    	set_cursor(0,2);
    	lcd_string("V3.3 (c)turi 2010");

		delay_ms(1500);
	}

	//PotiEn liegt auf Masse
	if ( ! (PINB & (1 << PotiEn))) 
	{
	 	//Kanle ADC einmalig lesen, um Startwerte zu setzen
		ADCval = ReadChannel(5);ADCvalC4=ADCval; /* MUX-Bits auf 0b0000 -> Channel 4 */
		ADCval = ReadChannel(3);ADCvalC3=ADCval; /* MUX-Bits auf 0b0000 -> Channel 3 */
		ADCval = ReadChannel(2);ADCvalC2=ADCval; /* MUX-Bits auf 0b0000 -> Channel 2 */
		ADCval = ReadChannel(1);ADCvalC1=ADCval; /* MUX-Bits auf 0b0000 -> Channel 1 */
		ADCval = ReadChannel(0);ADCvalC0=ADCval; /* MUX-Bits auf 0b0000 -> Channel 0 */
	}

	//EEPROM lesen und Werte setzen
	red_val=eeprom_read_word(&ee_pwm1);
	green_val=eeprom_read_word(&ee_pwm2);
	blue_val=eeprom_read_word(&ee_pwm3);
	bri_val=eeprom_read_word(&ee_pwm4);
	sat=eeprom_read_word(&ee_pwm5);
	hue=eeprom_read_word(&ee_pwm6);
	iProg=eeprom_read_byte(&ee_pwm7);
	speed=eeprom_read_word(&ee_pwm8);
	white_val=eeprom_read_word(&ee_pwm9);
	Strobo_pause=eeprom_read_word(&ee_pwm10);
	Strobo_on=eeprom_read_word(&ee_pwm11);
	DMXStartAddress=eeprom_read_word(&ee_pwm12);

	bri=pgm_read_word(bri_table_8+bri_val);

	//LCD lschen
	if ( ! (PINB & (1 << LCDEn))) 
	{
		lcd_clear();lcd_home();
	}

	//Hauptschleife
   	while(1) 
	{
			                
		   if (-1 == rc5.flip) //keine FB - Potiabfrage
		   {	//keine FB-Taste gedrckt: Kanle 0-3,5 ADC lesen
				//wenn PotiEn auf Masse liegt
				if ( ! (PINB & (1 << PotiEn)))	
				{
					ADCval = ReadChannel(5); 
					//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
					if(ADCvalC4<2) ADCdiff=ADCvalC4; else ADCdiff=2;
					if ((ADCval > ADCvalC4+2) || (ADCval < ADCvalC4-ADCdiff))
			 		{ 
						//Helligkeit gesamt ber Tabelle logarithmisch
						white_val=ADCvalC4/4; //0..255
						ADCvalC4=ADCval;
					}				
					ADCval = ReadChannel(3); 
					//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
					if(ADCvalC3<2) ADCdiff=ADCvalC3; else ADCdiff=2;
					if ((ADCval > ADCvalC3+2) || (ADCval < ADCvalC3-ADCdiff))
			 		{ 
						//Helligkeit gesamt ber Tabelle logarithmisch
						ADCvalC3=ADCval;
						bri_val=ADCvalC3/16;  //0..63
					}				
					ADCval = ReadChannel(2); 
					//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
					if(ADCvalC2<2) ADCdiff=ADCvalC2; else ADCdiff=2;
					if ((ADCval > ADCvalC2+2) || (ADCval < ADCvalC2-ADCdiff))
					{
						//ADC-Wert speichern
						ADCvalC2=ADCval;
						if ((iProg==7) || (iProg==8))
							blue_val=ADCvalC2/4; //0..255
						else if ((iProg==9))
							speed = ADCvalC2/2; //0..512
						else if ((iProg==4))
							Strobo_pause = ADCvalC2/2; //0..512
					}
					ADCval = ReadChannel(1); 
					//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
					if(ADCvalC1<2) ADCdiff=ADCvalC1; else ADCdiff=2;
					if ((ADCval > ADCvalC1+2) || (ADCval < ADCvalC1-ADCdiff))
					{
						//ADC-Wert speichern
						ADCvalC1=ADCval;
						if ((iProg==7) || (iProg==8))
							green_val=ADCvalC1/4; //0..255
						else if ((iProg==6))
							hue = ADCvalC1*3/4  ; //0..765
						else if ((iProg==4))
						{
							Strobo_on = ADCvalC1/2; //0..512
							if (Strobo_on<19) Strobo_on=10;
						}
					}
					ADCval = ReadChannel(0); 
					//Differenz zum vorhergehenden Wert? -> Poti wurde verndert
					if(ADCvalC0<2) ADCdiff=ADCvalC0; else ADCdiff=2;
					if ((ADCval > ADCvalC0+2) || (ADCval < ADCvalC0-ADCdiff))
					{
						//ADC-Wert speichern
						ADCvalC0=ADCval;
						if ((iProg==7) || (iProg==8))
							red_val=ADCvalC0/4; //0..255
						else if ((iProg==6) || (iProg==9))
							sat = ADCvalC0/8;  //0..128
					}
				} //PB6 auf Masse
				//DMX verarbeiten in Prog. 5, wenn keine FB gedrckt
				if(iProg==5)
				{
					bri_val=DmxRxField[0]/4;
					red_val=DmxRxField[1];
					green_val=DmxRxField[2];
					blue_val=DmxRxField[3];
					white_val=DmxRxField[4];
					//fr weitere Modi Umrechnung erforderlich
					//Mehrfachbelegung der Werte
					if (nDMXmode!=FADE1)
					{
						hue=DmxRxField[1]*3;
						speed=DmxRxField[3];
					}
					sat=DmxRxField[2]/2;
					//fr Strobo spezielle Begrenzung
					Strobo_on=DmxRxField[1];if (Strobo_on<10) Strobo_on=10;
					Strobo_pause=DmxRxField[2]*2;
					if (DmxRxField[5]<11) nDMXmode=RGB; 
					else if ((DmxRxField[5]>10) && (DmxRxField[5]<21)) nDMXmode=HSB;
					else if ((DmxRxField[5]>20) && (DmxRxField[5]<31)) nDMXmode=STROBO1;
					else if ((DmxRxField[5]>30) && (DmxRxField[5]<41)) nDMXmode=STROBO2;
					else if ((DmxRxField[5]>40) && (DmxRxField[5]<51)) nDMXmode=FADE1;
				}

		   }  //keine FB - Potiabfrage
		   else							
		   {	//FB verarbeiten
		      	/* Ja, dann rc5.code merken und evtl. rc5.addr */
		      	code = rc5.code;
		      	addr = rc5.addr;
				if (addr==DeviceType)		//TV-Modus etc.
				{
					switch (code)
					{
					case 12: { 	
								code=0;					
								if (LEDaus==1) //ausschalten
								{
									bri_val_old = bri_val;
									bri_val=0; 
									LEDaus=0;

								}
								else				// wieder auf vorhergehenden Wert einschalten
								{
									bri_val=bri_val_old; 
									bri_val_old = 0;
									LEDaus=1;
								}
								_delay_ms(500);
								break;
							}    // Aus/wieder ein

		    		case 60: { 	if (iProg==7) { white_val=0; }
								if (iProg==6) { hue=0;white_val=0; } //rot
								if (iProg==8) { red_val=255;green_val=0;blue_val=0; }				
								break;
							}    // Record

		    		case 46: { 	if (iProg==7) { bri_val=63;white_val=255; red_val=0;green_val=0;blue_val=0; }
								if (iProg==6) { hue=255;white_val=0; } //grn
								if (iProg==8) { red_val=0;green_val=255;blue_val=0;  }
								break;
							}    // back

		    		case 43: { 	if (iProg==7) { red_val=0;green_val=0;blue_val=0;white_val=0; }
								if (iProg==8) { red_val=0;green_val=0;blue_val=0;white_val=255; } //nur ww
								if (iProg==6) { hue=180;white_val=120; }
								if (iProg==9) { white_val=0; }					
								break;
							}    // Play

		    		case 44: { 	if (iProg==7) { bri_val=20;red_val=255;green_val=255;blue_val=245;white_val=255; }
								if (iProg==6) { hue=450;white_val=255; }  //xxx + ww
								if (iProg==8) { red_val=255;green_val=255;blue_val=160;white_val=255;  }					
								if (iProg==9) { white_val=60; }					
								break;
							}    // Stop

		    		case 56: { 	if (iProg==7) { red_val=0;green_val=0;blue_val=0; }
								if (iProg==6) { hue=382; white_val=0;  } 
								if (iProg==8) { red_val=255;green_val=255;blue_val=255; }
								if (iProg==9) { speed=10; Strobo_pause=50; Strobo_on=50; }	//Rcksetzen Werte
								break;
							}    // exit

		    		case 42: { 	if (key==0)
								{	//Speichern der akt. Werte eingeleitet
									eeprom_write_word(&ee_pwm1,red_val ); //rot speichern
									eeprom_write_word(&ee_pwm2,green_val ); //grn speichern
									eeprom_write_word(&ee_pwm3,blue_val ); //blau speichern
									eeprom_write_word(&ee_pwm4,bri_val ); //akt. Helligkeit speichern
									eeprom_write_word(&ee_pwm5,sat ); //sat speichern
									eeprom_write_word(&ee_pwm6,hue ); //hue speichern
									eeprom_write_byte(&ee_pwm7,iProg ); //Programm speichern
									eeprom_write_word(&ee_pwm8,speed ); //speed speichern
									eeprom_write_word(&ee_pwm9,white_val ); //speed speichern
									eeprom_write_word(&ee_pwm10,Strobo_pause ); //Strobo_pause speichern
									eeprom_write_word(&ee_pwm11,Strobo_on ); //Strobo_on speichern
									eeprom_write_word(&ee_pwm12,DMXStartAddress ); //DMX Startadresse speichern
									//eeprom_busy_wait();
									//Besttigung kurz blinken
									LEDw = 20;_delay_ms(100);
									LEDw = 160;_delay_ms(100);
									LEDw = 20;_delay_ms(100);
									LEDw = 160;_delay_ms(100);
									_delay_ms(20);
									key=9;
								}
								else
								{	if (iProg==7) { bri_val=63;red_val=255;green_val=255;blue_val=245;white_val=0; }
									if (iProg==6) { hue=509;white_val=0;  } //blau
									if (iProg==8) { red_val=0;green_val=0;blue_val=255;}
								}
								break;
							}    // forward

		    		case 53: { 	if (iProg==7) { bri_val=40;red_val=255;green_val=255;blue_val=245;white_val=255; }
								if (iProg==6) { hue=127; white_val=180; }
								if (iProg==8) { red_val=255;green_val=0;blue_val=255;white_val=0; }
								if (iProg==9) { white_val=120; }					
								break;
							}    // Pause

		    		case 13: { 	if (iProg==7) { bri_val=63;red_val=255;green_val=255;blue_val=250;white_val=255; }
								if (iProg==6) { hue=660; white_val=200; }
								if (iProg==8) { red_val=255;green_val=255;blue_val=0;white_val=0; }
								if (iProg==9) { white_val=255; }					
								break;
							}    // Mute

		    		case 16:// Vol + in Abhngigkeit Zifferntaste
					 		{ 
								if (key==1)
								{
									if ((iProg==7) || (iProg==8))
									{
										red_val+=5;
										if (red_val > 255) red_val = 255;
									}
									else if ((iProg==6) || (iProg==9))
									{
										sat+=4;
										if (sat > 127) sat = 127;
									}
								}
								if (key==2)
								{
									if ((iProg==7) || (iProg==8)) 
									{
										green_val+=5;
										if (green_val > 255) green_val = 255;
									}
									else if (iProg==6)
									{
										hue+=5;
										if (hue > 764) hue = 764;
									}
									else if (iProg==4)
									{
										Strobo_pause+=5;
										if (Strobo_pause > 512) Strobo_pause = 512;
									}
								}
								if (key==3)
								{
									if ((iProg==7) || (iProg==8))
									{
										blue_val+=5;
										if (blue_val > 255) blue_val = 255;
									}
									else if (iProg==9)
									{
										speed-=5;
										if (speed < 10) speed = 10;
									}
									else if (iProg==4)
									{
										Strobo_on+=5;
										if (Strobo_on > 512) Strobo_on = 512;
									}
								}
								if (key==5)
								{
										white_val+=5;
										if (white_val > 255) white_val = 255;
								}
								break;
							}    
		    		case 17:// Vol - in Abhngigkeit Zifferntaste
							 { 
								if (key==1)
								{
									if ((iProg==7) || (iProg==8)) 
									{
										red_val-=5;
										if (red_val < 0) red_val = 0;
									}
									else if ((iProg==6) || (iProg==9))
									{
										sat-=4;
										if (sat < 0) sat = 0;
									}
								}
								if (key==2)
								{
									if ((iProg==7) || (iProg==8)) 
									{
										green_val-=5;
										if (green_val < 0) green_val = 0;
									}
									else if (iProg==6)
									{
										hue-=5;
										if (hue < 0) hue = 0;
									}
									else if (iProg==4)
									{
										Strobo_pause-=5;
										if (Strobo_pause < 10) Strobo_pause = 10;
									}
								}
								if (key==3)
								{
									if ((iProg==7) || (iProg==8))
									{
										blue_val-=5;
										if (blue_val < 0) blue_val = 0;
									}
									else if (iProg==9)
									{
										speed+=5;
										if (speed > 200) speed = 200;
									}
									else if (iProg==4)
									{
										Strobo_on-=5;
										if (Strobo_on < 10) Strobo_on = 10;
									}
								}
								if (key==5)
								{
										white_val-=5;
										if (white_val < 0) white_val = 0;
								}
								break;

							}    
		    		case 32: { // Ch + 
									bri_val+=1;
									if (bri_val > 63) bri_val = 63;
								break;
							}    
		    		case 33: { // Ch - 
									bri_val-=1;
									if (bri_val < 0) bri_val = 0;
								break;
							}    

					case 59: {	//Taste Men zum Umschalten LCD-Anzeige
								if(bLCD==0) bLCD=1; else bLCD=0; 
								_delay_ms(500);
								break;
							}
					case 11: {	//Taste P/C als Prfix fr Einzelfarben ndern (key 1,2,3,5)
								if(bEFarbe==0) bEFarbe=1; else bEFarbe=0; 
								_delay_ms(500);
								break;
							}

					case 1:	{	key=1; break;}		//Farbe 1 rot
					case 2:	{	key=2; break;}		//Farbe 2 grn
					case 3:	{	key=3; break;}		//Farbe 3 blau
					case 5:	{	if (bEFarbe==1) key=5;
								else iProg=5; 
								break;}				//Farbe 4 wei oder Prog 5
					case 4:	{	iProg=4;break;}		//Prog Strobo	
					case 6:	{	iProg=6; break;}	//Prog HSB manuell
					case 7:	{	iProg=7; break;}	//Prog wei	
					case 9: {	iProg=9; break;}	//Prog HSB-Fading
					case 8: {	iProg=8; break;}	//Prog RGB manuell
					case 0: {	key=0; break;}	//Speichern einleiten
		    		//default: ;    

					} //switch (code)


				} //addr==0

		      	/* auf naechstes Zeichen warten */
		      	rc5.flip = -1;

			} //FB-Verarbeitung
		

		//Helligkeit ber Lookup-Tabelle berechnen
		bri=pgm_read_word(bri_table_8+bri_val);  //0..255

		//HSB-Fading-Programm
		if ((iProg==9) || ((iProg==5) && (nDMXmode==FADE1)))		
		{		   
			hue++;
			//Verzgerung
			_delay_ms(speed);

			if (! (PINB & (1 << LCDEn)) )
			{
				//LCD ausgeben
				char Buffer[10]; // in diesem {} lokal
				set_cursor(11,1);
       			itoa( speed, Buffer, 10 ); 
       			lcd_string( "D:" );
				lcd_string( Buffer );		
				lcd_string("  ");
			}
		}
		//HSB-Werte aus FB bzw. DMX interpretieren 
		if ( (iProg==6) || (iProg==9) || ((iProg==5) && (nDMXmode==HSB)) 
									  || ((iProg==5) && (nDMXmode==FADE1)) )
		{		   
			if (hue > 764) hue = 0;

			if (hue < 255) {
			   red_val = (10880 - sat * (hue - 170)) >> 7;
			   green_val = (10880 - sat * (85 - hue)) >> 7;
			   blue_val = (10880 - sat * 85) >> 7;
			}
			else if (hue < 510) {
			   red_val = (10880 - sat * 85) >> 7;
			   green_val = (10880 - sat * (hue - 425)) >> 7;
			   blue_val = (10880 - sat * (340 - hue)) >> 7;
			}
			else {
			   red_val = (10880 - sat * (595 - hue)) >> 7;
			   green_val = (10880 - sat * 85) >> 7;
			   blue_val = (10880 - sat * (hue - 680)) >> 7;
			}
			//LCD ausgeben
			if (! (PINB & (1 << PB7)) )
			{
				char Buffer[10]; // in diesem {} lokal
				set_cursor(0,1);
       			itoa( hue, Buffer, 10 ); 
       			lcd_string( "H:" );
				lcd_string( Buffer );
				lcd_string("    ");
				set_cursor(6,1);
       			itoa( sat, Buffer, 10 ); 
       			lcd_string( "S:" );
				lcd_string( Buffer );
				lcd_string("  ");
			}						
		}

		//Strobo-Programm wei nur fr DMX
		if ( ((iProg==5) && (nDMXmode==STROBO2)) )
		{	//Stroboskop je Durchlauf einen Teil ausgeben
			red=green=blue=white=((bri + 1) * 255) >> 8;
			LEDr = red;
	      	LEDg = green;
	      	LEDb = blue;
	      	LEDw = white;
			delay_ms(Strobo_on);
			red=green=blue=white= 0;
			LEDr = red;
	      	LEDg = green;
	      	LEDb = blue;
	      	LEDw = white;
			delay_ms(Strobo_pause);
		}

		//Strobo-Programm RGBW fr Fernbedienung und DMX
		if ( (iProg==4)  || ((iProg==5) && (nDMXmode==STROBO1)) )
		{	//Stroboskop je Durchlauf einen Teil ausgeben
			switch (iRun)
			{
				case 0:
				{
					red=((bri + 1) * 255) >> 8;
					green=0;blue=0;white=0;
					LEDr = red;
			      	LEDg = green;
			      	LEDb = blue;
			      	LEDw = white;
					delay_ms(Strobo_on);
					red = 0;
					LEDr = red;
			      	LEDg = green;
			      	LEDb = blue;
			      	LEDw = white;
					delay_ms(Strobo_pause);
					break;
				}
				case 1:
				{
					green = ((bri + 1) * 255) >> 8;
					LEDr = red;
			      	LEDg = green;
			      	LEDb = blue;
			      	LEDw = white;
					delay_ms(Strobo_on);
					green = 0;
					LEDr = red;
			      	LEDg = green;
			      	LEDb = blue;
			      	LEDw = white;
					delay_ms(Strobo_pause);
					break;
				}
				case 2:
				{
					blue = ((bri + 1) * 255) >> 8;
					LEDr = red;
			      	LEDg = green;
			      	LEDb = blue;
			      	LEDw = white;
					delay_ms(Strobo_on);
					blue = 0;
					LEDr = red;
			      	LEDg = green;
			      	LEDb = blue;
			      	LEDw = white;
					delay_ms(Strobo_pause);
					break;
				}
				case 3:
				{
					white = ((bri + 1) * 255) >> 8;
					LEDr = red;
			      	LEDg = green;
			      	LEDb = blue;
			      	LEDw = white;
					delay_ms(Strobo_on);
					white = 0;
					LEDr = red;
			      	LEDg = green;
			      	LEDb = blue;
			      	LEDw = white;
					delay_ms(Strobo_pause);
					break;
				}
			} //switch
			iRun++; if(iRun>3) iRun=0;
		}
		//fr alle auer Prog 4 ausgeben
		else //if ( (iProg>4)  || (nDMXmode==RGB) )

		{
			// Ausgabe der Werte
			red = ((bri + 1) * red_val) >> 8;
			green = ((bri + 1) * green_val) >> 8;
			blue = ((bri + 1) * blue_val) >> 8;		   
			white = ((bri + 1) * white_val) >> 8;		   
			LEDr = red;
			LEDg = green;
			LEDb = blue;		
			LEDw = white;		
		}


		if (((iProg==7) || (iProg==8) || ( ((iProg==5)) && (nDMXmode==RGB) )) && (! (PINB & (1 << LCDEn))) )
		{		   
			//LCD ausgeben
			char Buffer[10]; // in diesem {} lokal
			set_cursor(0,1);
   			itoa( red, Buffer, 10 ); 
   			lcd_string( "R:" );
			lcd_string( Buffer );
			lcd_string("    ");
			set_cursor(6,1);
   			itoa( green, Buffer, 10 ); 
   			lcd_string( "G:" );
			lcd_string( Buffer );
			lcd_string("    ");
			set_cursor(11,1);
   			itoa( blue, Buffer, 10 ); 
   			lcd_string( "B:" );
			lcd_string( Buffer );		
			lcd_string("    ");
		}

		//LCD-Ausgabe Programm-Nummer und Fernbedienungsdaten oder Infos
		if (! (PINB & (1 << LCDEn)) )
		{
			if (bLCD==1)
			{
				char Buffer[10]; // in diesem {} lokal
				set_cursor(0,2);
	   			itoa( iProg, Buffer, 10 ); 
	   			lcd_string( "P:" );
				lcd_string( Buffer );		
	   			lcd_string( "   " );
				set_cursor(4,2);
	   			itoa( addr, Buffer, 10 ); 
	   			lcd_string( "Adr:" );
				lcd_string( Buffer );		
	   			lcd_string( "   " );
				set_cursor(11,2);
	   			itoa( code, Buffer, 10 ); 
	   			lcd_string( " C:" );
				lcd_string( Buffer );
	   			lcd_string( "   " );
			}
			else
			{
				char Buffer[10]; // in diesem {} lokal
				set_cursor(0,2);
	   			itoa( iProg, Buffer, 10 ); 
	   			lcd_string( "P:" );
				lcd_string( Buffer );		
	   			lcd_string( "   " );
				set_cursor(4,2);
	   			itoa( bri, Buffer, 10 ); 
	   			lcd_string( "Bri:" );
				lcd_string( Buffer );		
	   			lcd_string( "   " );
				set_cursor(11,2);
	   			itoa( key, Buffer, 10 ); 
	   			lcd_string( "key:" );
				lcd_string( Buffer );
	   			lcd_string( "   " );
			}
		}

   }    //while                     
 
   return 0;                 
}



ISR(PCINT0_vect)       
{
	cli();
	{
		if (iTaste==0)
		{
			iProg=iProg+1;
			if (iProg > 9) iProg=4;
			iTaste=1;
		}
		else
			iTaste=0;
		delay_ms(100);
	}
	sei();
}



