/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX512 reception library
* Version		: v1.3
* Last updated	: 13.04.09
* Target		: Transceiver Rev.3.01 [ATmega8515]
* Clock			: 8MHz, 16MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/


#include "lib_dmx_in.h"


// ********************* local definitions *********************

//enum {IDLE, BREAK, STARTB, STARTADR};			//DMX states (nur fr Original-ISR)

volatile uint8_t gDmxState;
volatile uint8_t flag;						// new data available


// *************** DMX Reception Initialisation ****************
void init_DMX_RX(void)
{

//DIP-Schalter individuell anpassen, fr RGBW-Controller keine verwenden
#ifdef USE_DIP
DDRC   = 0;										//set up DIPs
PORTC  = 0xFF;
DDRD  &= ~((1<<PD2)|(1<<PD1));
PORTD |=   (1<<PD2)|(1<<PD1);
#endif


#ifdef __AVR_ATmega8515__
DDRD  |= (1<<2);
PORTD &= ~(1<<2);								
UBRRH  = 0;										//enable reception
UBRRL  = ((F_OSC/4000)-1);						//250kbaud, 8N2
UCSRC  = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
UCSRB  = (1<<RXEN)|(1<<RXCIE);
gDmxState= IDLE;
#endif

#ifdef __AVR_ATmega168__
/* Set baud rate */
UBRR0H = UBRR_VAL >> 8;
UBRR0L = UBRR_VAL & 0xFF;

/* Enable receiver */
UCSR0B = (1<<RXEN0)|(1<<RXCIE0);
/* Set frame format: 8data, 2stop bit */
UCSR0C = (1<<USBS0)|(1<<UCSZ00)|(1<<UCSZ01);//|(1<<UMSEL00);
//gDmxState= IDLE;
#endif
}



// ************* get DMX start address **************
void get_dips(void)
{
#ifdef USE_DIP
uint16_t Temp= (0xFF-PINC);						//invert DIP state
if (!(PINE &(1<<2)))
	{
	Temp += 256;								//9th bit
	}
if (Temp != 0)
	{
	DMXStartAddress= Temp;
	if (!(UCSRB &(1<<RXCIE)))					//if receiver was disabled -> enable and wait for break
		{
		gDmxState= IDLE;
		UCSRB |= (1<<RXCIE);
		}
	}
else
	{
	UCSRB &= ~(1<<RXCIE);						//disable Receiver if start address = 0
	uint8_t i;
	for (i=0; i<sizeof(DmxRxField); i++)
		{
		DmxRxField[i]= 0;
		}
	}
#endif 
}


/*
// *************** DMX Reception ISR ****************
ISR (USART_RX_vect)
{
static  uint16_t DmxCount;
		uint8_t  USARTstate= UCSR0A;				//get state before data!
		uint8_t  DmxByte   = UDR0;				//get data
		uint8_t  DmxState  = gDmxState;			//just load once from SRAM to increase speed
 
if (USARTstate &(1<<FE0))						//check for break
	{
	UCSR0A &= ~(1<<FE0);							//reset flag (necessary for simulation in AVR Studio)
	DmxCount =  DmxAddress;						//reset channel counter (count channels before start address)
	gDmxState= BREAK;
	}

else if (DmxState == BREAK)
	{
	if (DmxByte == 0) gDmxState= STARTB;		//normal start code detected
	else			  gDmxState= IDLE;
	}
	
else if (DmxState == STARTB)
	{
	if (--DmxCount == 0)						//start address reached?
		{
		DmxCount= 1;							//set up counter for required channels
		DmxRxField[0]= DmxByte;					//get 1st DMX channel of device
		gDmxState= STARTADR;
		}
	}

else if (DmxState == STARTADR)
	{
	DmxCount++;
	DmxRxField[DmxCount]= DmxByte;			//get channel
	if (DmxCount >= sizeof(DmxRxField)) 		//all ch received?
		{
		gDmxState= IDLE;						//wait for next break
		}
	}
	DMXInfo=DmxCount;							

}

*/

//neue ISR aus WWW geladen, da Original nicht arbeitet

ISR(USART_RX_vect) {
	static uint16_t dmx_channel, dmx_base;
	uint8_t status, data;
	int16_t index;
 
	// read UART data, also clears interrupt flag        	
	status = UCSR0A;
	data = UDR0;

	if (status & (1<<FE0)) {					// frame error
		if (data==0) {						// break -> DMX Reset
			dmx_channel=0;
			dmx_base = DMXStartAddress;		//DMX-Startadresse aus EEPROM		
			flag=1;							// trigger update
		}
		else								// rx error
			dmx_channel++;
	}
	else {
		index = dmx_channel-dmx_base;
		if (index>=0 && index<dmx_size)
			DmxRxField[index]=data;
		dmx_channel++;
	}
}

